/*
 * Copyright (c) 2002 Tomas Svensson <ts@unix1.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <err.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <netdb.h>

#include "lamefwd.h"
#include "config.h"

int getconfig(struct fwdent *fwdent, int fwdentmax)
{
	FILE 	*fin;
	char 	line[80], *ptr;
	char 	**ap, *arg[10], *inputstring, *ep;
	int	args, ent;
	struct sockaddr_in saddr;
        struct hostent *hptr;

	if ( (fin = fopen("lamefwd.cfg", "r")) == NULL)
		errx(1, "can't open config file\n");

	ent = 0;

	while (!feof(fin)) {
		bzero(line, sizeof(line));
		fgets(line, sizeof(line), fin);	
		if ( (ptr = memchr(line,'\r',sizeof(line))) != NULL)
			*ptr='\0';
		if ( (ptr = memchr(line,'\n',sizeof(line))) != NULL)
			*ptr='\0'; 
		inputstring = &line[0];
		args = 0;
           	for (ap = arg; (*ap = strsep(&inputstring, " \t")) != NULL;)
                	if (**ap != '\0') {
				args++;
                           	if (++ap >= &arg[10])
                                   break;
			}
		if (args == 4 && *arg[0] != '#') {
			fwdent[ent].localport = (unsigned int)strtol(arg[0],
			    &ep, 10);
			if (fwdent[ent].localport <= 0 || *ep != '\0')
				errx(1, "illegal local port number -- %s",
				    arg[0]);
			fwdent[ent].remoteport = (unsigned int)strtol(arg[2],
			    &ep, 10);
			if (fwdent[ent].remoteport <= 0 || *ep != '\0')
				errx(1, "illegal remote port number -- %s",
				    arg[2]);

			bzero(&saddr, sizeof(saddr));
			if ((hptr = gethostbyname(arg[1])) == NULL)
				errx(1, "can't resolve %s", arg[1]);
			memcpy(&saddr.sin_addr, hptr->h_addr,
			    sizeof(saddr.sin_addr));
			saddr.sin_port = htons(fwdent[ent].remoteport);
			saddr.sin_family = PF_INET;
			if (strcasecmp(arg[3], "tcp") == 0)
				fwdent[ent].proto = FWD_TCP;
			else if (strcasecmp(arg[3], "udp") == 0)
				fwdent[ent].proto = FWD_UDP;
			else
				errx(1, "unknown protocol: %s", arg[3]);
			memcpy(&fwdent[ent].sin, &saddr, sizeof(fwdent[ent].sin));
			ent++;
		}
	}
	fclose(fin);

	return ent;
}

