/*
 * Copyright (c) 2002 Tomas Svensson <ts@unix1.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "lamefwd.h"
#include "config.h"
#include "io.h"
#include "network2.h"

#define MAX_CFG_ENTRIES 10

int debug;

int main(int argc, char * const * argv)
{
	int	ents;
	int	cfg_max_users;
	struct fwdent fwdent[MAX_CFG_ENTRIES];
	struct ud *ud;

	cfg_max_users = 30;
	debug = 0;

	if ( (ud = calloc(cfg_max_users, sizeof(struct ud))) == NULL)
		errx(1, "calloc failed");

	ents = getconfig(&fwdent[0], MAX_CFG_ENTRIES);
	if (ents == 0)
                errx(1, "No forwardable ports defined in the config file");

	fprintf(stderr,
	    "lamefwd v0.6 (c) 2002-2005 Tomas Svensson <ts@unix1.net>\nMax TCP connections: %d\nPorts forwarded: %d\n",
	    cfg_max_users, ents);
	#ifndef __CYGWIN__
#ifdef __HAVE_DAEMON
	if (!debug)
		daemon(0 ,0);
#else
	if (!debug) {
		pid_t childpid;
		if ( (childpid = fork()) < 0) 
				sys_err("fork()");
    	else if (childpid != 0) {
			fprintf(stderr, "Running as process %u\n", (unsigned int)childpid);
			exit(0); /* parent */
		}
		(void)setsid();
		chdir("/");
	}
#endif /* !HAVE_DAEMON */
#endif /* !__CYGWIN__ */
	io_setup(&fwdent[0], ents, ud, cfg_max_users);
	io_main(&fwdent[0], ents, ud, cfg_max_users);
	
	return 0;
}
