/*
 * Copyright (c) 2002 Tomas Svensson <ts@unix1.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <string.h>

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include "network2.h"

extern int debug;

void sys_err(const char *err){
	perror(err);
  	exit(1);
}

int setup_listen(int backlog, unsigned int port, int type)
{
	struct sockaddr_in sin;
	int	flags, sock, sockopt;
	
        memset(&sin, 0, sizeof(sin));
	sin.sin_family = PF_INET;
	sin.sin_addr.s_addr = htonl(INADDR_ANY);
	sin.sin_port = htons(port);
	sock = socket(PF_INET, type, 0);
	if (sock < 0)
		sys_err("socket");
	sockopt = 1;
        if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &sockopt, sizeof
(sockopt)))
                sys_err("setsockopt-reuseaddr");
	if (bind(sock, (struct sockaddr *)&sin, sizeof(sin))) {
		perror("bind");
		exit(1);
	}
	if (type == SOCK_STREAM)
		if (listen(sock, backlog)) {
			perror("listen");
			exit(1);
		}
	flags = fcntl(sock, F_GETFL);

	fcntl(sock, F_SETFL, flags | O_NONBLOCK);
	
	if (debug)
		printf("listening to port %u on fd %d\n", port, sock);	
	return sock;
}

int setup_connect(int type, struct sockaddr *sa, socklen_t sa_len, int *result)
{
        int conn_fd, flags,  sockopt;

	conn_fd = socket(PF_INET, type, 0);
        if (conn_fd < 0)
                sys_err("setup_connect_socket");

       	flags = fcntl(conn_fd,F_GETFL);
       	fcntl(conn_fd, F_SETFL, flags | O_NONBLOCK);

	if (type == SOCK_STREAM) {
        	sockopt = 1;
        	if (setsockopt(conn_fd, SOL_SOCKET, SO_KEEPALIVE, &sockopt, sizeof(sockopt)))
                	sys_err("setsockopt-keepalive");
	}
	/*
        if (bind(conn_fd, sa, sa_len) < 0)
                perror("bind");
	*/

	if (connect(conn_fd, sa, sa_len) != 0) {
		if (errno == ECONNREFUSED) {
			*result = 3;
			if (debug)
				printf("connection refused when connecting to destination host\n");
		} else {
			if (errno != EINPROGRESS) {
				perror("connect");
				sys_err("connect");
			}
			*result = 1; /* Nonblocking operation */
		}
	} else
		*result = 0; /* Connect OK */
	
	return conn_fd;
}
